;************************************************************************
;*		This Program was written in an attempt to convey something		*
;*		useful to someone new to these devices. It will					*
;*		use timer0, and two of the I/O pins. It will continously		*
;*		toggle an LED depending on if a switch is pressed.				*
;*		The flash period is adjustable at compile time.					*
;*																		*
;*		The LED is connected to GP bit 5 (Pin 2)						*	
;*		The LED requires a 330R resistor in series with it,				*
;*																		*
;*										   LED							*	
;*		Cathode to Vss like this:	Vss |--|<--o-/\/\/-o--o GP 5		*	
;*											\\	 330R					*	
;*																		*	
;*		The switch uses the weak pull-up internal to the PIC chip		*
;*		and therefore does not require an external pull-up resistor.	*
;*		The switch is connected between GP bit 3 (Pin 4) and Vss.		*
;*																		*
;*																		*
;*																		*
;*	If you see D'xx' that means the number is expressed as Decimal.		*	
;*	If you see H'xx' that means the number is expressed as Hexadecimal.	*
;*	If you see B'xxxxxxxx' that means the number is expressed as Binary.*
;*	If you see a ; that means whatever comes after that is a comment.	*
;*	I prefer using Hexadecimal, but will sometimes use Binary.			*
;*	Anything that starts at the far left of the page, but doesn't have	*
;*	a ; before it is classed as a label, a place name or marker if you	*
;*	like. Whenever you see F or W after a comma (,) it simply means		*
;*	put the result of the instruction or data in either the file		*
;*	register named before the comma	or put it into the W register.		*
;*																		*
;*
;*	If you are completely new to all of this stuff, then you will be	*
;*	feeling a little daunted reading this by now, but take heart, read	*
;*	the datasheet for the chip several times and absorb the little bits	*
;*	first, then read through this code several times and rationalise	*
;*	what you see in the datasheet and what you see here, it won't be	*
;*	long before you are writing a little add on for this code.			*
;*	Anyway I hope you will get something useful out of reading and		*
;*	playing with this code, I wish you well...							*
;*																		*
;*																		*
;*																		*
;*																		*
;*Project		:	Demo - Flashing LED									*
;*Author		:	tunedwolf@electro-tech-online						*
;*Version		:	1.00 beta											*
;*Target Device	:	PIC12C508											*
;*																		*
;*Copyright		:	 27th October 2006 Alastair McGloan				*
;*					This may be freely distributed and used for any		*
;*					non commercial purposes, providing it is			*
;*					distributed in it's complete format, including this	*
;*					header.												*
;*					It's aim is as an educational aid, I make no		*
;*					claim to it's fitness of purpose, and provide no	*
;*					warranty whatsoever. You agree by using this that	*
;*					it will not be used in life critical applications	*
;*					of any kind, and that I am not liable for any		*
;*					damages howsoever caused.							*
;*					 													*
;************************************************************************
;************************************************************************
;COMPILER DIRECTIVES ETC :
;************************************************************************

;This has been compiled and tested using MPlab ver 7.42
;and in the actual target processor.
;Obtain the datasheet from Microchip for the device you are using and
;READ IT FROM COVER TO COVER


	LIST P=12C508A
	#INCLUDE "P12C508A.INC"


;*** CONFIGURATION BITS GET SET HERE ***

;MCLR WILL BE GENERAL I/O
;CODE PROTECTION WILL BE - OFF
;WATCHDOG TIMER WILL BE - OFF
;OSCILLATOR WILL BE - INTERNAL 4Mhz


	__FUSES _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC 


;*** DEVICE ID GETS SET HERE ***

;SET ID LOCS

;JUST FOR THE SAKE OF PUTTING A CODE REVISION NUMBER SOMEWHERE

	__IDLOCS H'0001'

;************************************************************************
;USER RAM REGISTER EQUATES ETC :
;************************************************************************
;GENERAL PURPOSE RAM REGISTERS START AT ADDRESS H'07' (FROM DATASHEET)


TEMP		EQU	H'07'			;THIS REGISTER IS USED TO DUMP TEMPORARY STUFF INTO

TIME		EQU	H'08'			;WE USE THIS REGISTER TO HOLD OUR PREVIOUS TIMER COUNT VALUE

COUNT		EQU	H'09'			;WE USE THIS REGISTER TO HOLD OUR FLASH RATE PERIOD COUNT

;************************************************************************
;************************************************************************
;CONSTANTS AND USER EQUATES ETC:
;************************************************************************
;HERE YOU CAN ADJUST THE FLASH RATE BY CHANGING THE VALUE OF RATE
;THIS MERELY SETS OUR COUNTER VALUE HIGHER OR LOWER.

;WITH THE DEFAULT SETTING OF 7 THE FLASH RATE WILL BE APPROX 0.5 SECS
;TRY DIFFERENT VALUES IN HERE TO SEE WHAT HAPPENS - ANY VALUE BETWEEN 0 AND 255 (H'00' - H'FF')

;THESE VALUES ONLY GIVE APPROXIMATE TIMINGS, NOT PARTICULARLY ACCURATE, BUT FINE FOR OUR
;EXAMPLE HERE. IF YOU REQUIRE MORE ACCURACY THEN YOU WILL NEED TO USE AN EXTERNAL XTAL OR RESONATOR.


;OTHER SETTINGS THAT MIGHT BE USEFUL WOULD BE:-
;0.25 SECS APPROX	= H'04'
;1 SEC APPROX		= H'0F'

RATE		EQU	H'07'		;USED TO SET THE LED FLASH RATE - THIS IS THE VALUE YOU WOULD
							;CHANGE TO MAKE THE FLASH FASTER OR SLOWER
;************************************************************************
;************************************************************************
;GPIO PIN ASSIGNMENT :
;************************************************************************
;
SPARE_0		EQU	H'00'		;SPARE OUTPUT (PIN 7) GP-0
SPARE_1		EQU	H'01'		;SPARE OUTPUT (PIN 6) GP-1
SPARE_2		EQU	H'02'		;SPARE OUTPUT (PIN 5) GP-2
SW1			EQU	H'03'		;OUR SWITCH INPUT (PIN 4) GP-3
SPARE4		EQU	H'04'		;SPARE OUTPUT (PIN 3) GP-4
LED1		EQU	H'05'		;OUTPUT TO OUR LED (PIN 2) GP-5
;************************************************************************
;************************************************************************
;RESET VECTOR :
;************************************************************************

	ORG		H'00'				;START CODE HERE WHEN RESET OCCURS
	GOTO	INIT				;INITIALISE DEVICE AND RUN PROGRAM

;************************************************************************
;************************************************************************
;LED FLASH ROUTINE:
;************************************************************************

UPD_LED:
	MOVF	TIME,W		;GET RTC VALUE THAT WE STORED THE LAST TIME WE WERE HERE
	SUBWF	TMR0,W		;SUBTRACT THAT FROM THE CURRENT RTC VALUE
	BTFSS	STATUS,C	;HAS THE RTC OVERFLOWED YET ?
	GOTO	UPTIM		;YES - CHECK OUR PERIOD COUNTER
	ADDWF	TIME,F		;NO - JUST UPDATE OUR TIMER VALUE (ADD OUR CURRENT VALUE TO OUR OLD ONE)
	RETLW	H'00'		;AND RETURN TO THE CALLING ROUTINE (IN OUR CASE THE MAIN PROGRAM LOOP)

UPTIM:
	ADDWF	TIME,F		;UPDATE OUR TIMER VALUE (ADD THE CURRENT TIMER VALUE TO OUR OLD ONE)
	DECFSZ	COUNT,F		;KEEP COUNT FOR OUR FLASH RATE - IF IT HAS REACHED ZERO THEN THE NEXT INSTRUCTION
						;WILL BE SUBSTITUTED FOR A NOP - IF IT IS NOT ZERO THEN THE NEXT INSTRUCTION WILL BE EXECUTED
						;SO WITH THAT IN MIND - HAS OUR COUNTER FOR OUR FLASH RATE REACHED ZERO YET ?

	RETLW	H'00'		;NO - JUST RETURN TO THE CALLING ROUTINE (IN OUR CASE THE MAIN PROGRAM LOOP)

	MOVLW	RATE		;GET OUR FLASH RATE PERIOD FROM UP THERE ^ (WE SET IT EARLIER UP THERE RMEMEMBER ?)

	MOVWF	COUNT		;YES - SO RESET OUR FLASH RATE COUNTER VALUE FROM UP THERE ^ REMEMBER ?		

	BTFSS	GPIO,5		;TEST THE PORT PIN TO SEE IF THE LED IS CURRENTLY ON OR OFF ?
						;IF IT'S HIGH THE NEXT INSTRUCTION IS SUBSTITUTED BY A NOP AUTOMATICALLY
						;IF IT'S LOW THE NEXT INSTRUCTION WILL BE EXECUTED

	GOTO	LED_ON		;IT'S CURRENTLY OFF - SO TURN IT ON

LED_OFF:
	BCF		GPIO,5		;IT'S CURRENTLY ON - SO TURN IT OFF
	RETLW	H'00'		;ALL DONE - RETURN TO CALLING ROUTINE (IN OUR CASE THE MAIN PROGRAM LOOP)

LED_ON:
	BSF		GPIO,5
	RETLW	H'00'		;ALL DONE - RETURN TO CALLING ROUTINE (IN OUR CASE THE MAIN PROGRAM LOOP)
;************************************************************************
;************************************************************************
;MAIN CODE STARTS HERE :
;************************************************************************

;WE INITITALISE EVERYTHING AT THIS POINT - REMEMBER THAT "GOTO INIT" AT ORG H'00' ?
;THIS IS WHERE IT BROUGHT US TO, IT'S A MATTER OF CHOICE REALLY WHERE YOU PUT YOUR INITIALISATION STUFF
;I PERSONALLY PREFER TO PUT IT IMMEDIATELY BEFORE ANY TABLES, WHICH I NORMALLY PUT AT THE END OF THE MAIN
;BODY OF CODE - THIS GIVE YOU A LITTLE ADVANTAGE WHEN USING A LARGER MEMORY DEVICE - BECAUSE YOU CAN PUT
;ALL YOUR TABLES ON A HIGH PAGE WELL AWAY FROM YOUR MAIN CODE, YOU CAN ARRANGE THINGS SO THAT JUST ONE SET OF PAGING INSTRUCTIONS
;HAVE TO BE DEALT WITH IN ORDER TO ACCESS YOUR TABLES

;INITIALISATION STARTS HERE

INIT:
	MOVWF	OSCCAL				;UPDATE THE OSCILLATOR CALIBRATION WITH THE FACTORY SETTINGS
								;THE W REGISTER IS AUTOMATICALLY LOADED WITH THE CALIBRATION VALUE
								;UPON A RESET
	MOVLW	H'87'
	OPTION						;SET PRESCALER FOR /256 AND ASSIGN IT TO TIMER 0
								;SET WEAK PULL-UPS = ON
								;SET CLOCK SOURCE FOR TIMER TO USE INTERNAL INSTRUCTION CLOCK
								;SET CLOCK EDGE AS RISING EDGE FOR EXTERNAL CLOCK - WE AREN'T ACTUALLY USING THIS
								;THIS IS ONLY FOR AN EXTERNAL CLOCKING SCHEME


	MOVLW	H'08'
	TRIS	GPIO				;SET UP GP BIT 3 (PIN 4) AS INPUT
								;THE REST OF GP PORT AS OUTPUT (ON PIC12C508, GP-3 (PIN 4) IS INPUT ONLY ANYWAY)

	CLRF	GPIO				;SWITCH ALL OUTPUTS OFF


;AND ENDS HERE

;************************************************************************
;************************************************************************
;OUR MAIN PROGRAM STARTS HERE - THIS WILL CALL OUR LED ROUNTINE AS OFTEN AS WE WISH, AND DEPENDING ON WHAT VALUES
;WE HAVE SET FOR OUR FLASH RATE AND WHETHER THE SWITCH IS PRESSED, WILL FLASH OUR LED ON AND OFF AT THE RATE WE SET UP THERE ^.
;************************************************************************



MAIN_LOOP:
	
	BTFSC	GPIO,3				;TEST THE SWITCH INPUT - IF GP BIT 3 GOES LOW, I.E SWITCH PRESSED, THEN THE NEXT INSTRUCTION IS
								;SUBSTITUTED WITH A "NOP" AUTOMATICALLY - IF IT STAYS HIGH, I.E SWITCH IS NOT PRESSED, THEN OBVIOUSLY
								;IT WON'T, AND THE NEXT INSTRUCTION WILL BE EXECUTED
								;WITH THAT IN MIND - HAS THE SWITCH BEEN PRESSED YET ?

	GOTO	MAIN_LOOP			;NOPE - GO BACK AND TRY AGAIN

	MOVLW	H'0F'
	MOVWF	TEMP				;WE SHALL PUT A SMALL VALUE IN HERE SO THAT WE CAN PROVIDE A SMALL DELAY IN ORDER TO DEBOUNCE
								;THE SWITCH INPUT A LITTLE

DEBOUNCE:
	DECFSZ	TEMP,F				;COUNT DOWN - WHEN WE REACH ZERO THE NEXT INSTRUCTION WILL BE SUBSTITUTED WITH A "NOP"
								;AUTOMATICALLY - IF WE HAVEN'T REACHED ZERO, OBVIOUSLY IT WON'T BE, AND THE NEXT INSTRUCTION
								;WILL BE EXECUTED - WITH THAT IN MIND - HAVE WE REACHED ZERO YET ?
	
	GOTO	DEBOUNCE			;NO - GO ROUND OUR LOOP AGAIN UNTIL WE DO


	BTFSC	GPIO,3				;TEST THE SWITCH INPUT - IF IT IS LOW, I.E SWITCH IS PRESSED, THEN THE NEXT INSTRUCTION IS SUBSTITUTED
								;WITH A "NOP" AUTOMATICALLY - IF IT IS HIGH THEN OBVIOUSLY IT WON' BE AND THE NEXT INSTRUCTION WILL
								;BE EXECUTED - SO WITH THAT IN MIND - IS THE SWITCH STILL BEING PRESSED ?

	GOTO	MAIN_LOOP			;NO - START OVER AGAIN - THIS COULD JUST BE A GLITCH - OR PERHAPS THE SWITCH HAS NOT HAD ENOUGH
								;TIME TO SETTLE YET - SO WE NEED TO GIVE IT MORE TIME AND TRY AGAIN


	MOVLW	RATE				;THIS IS OUR FLASH RATE FROM UP THERE ^
	MOVWF	COUNT				;SET OUR COUNTER VALUE FOR OUR PERIOD TIMER

	MOVF	TMR0,W				;READ THE CONTENTS OF THE TIMER 0 AND SYNCRONISE OUR TIMER SO OUR FLASH RATE PERIOD ALWAYS
	MOVWF	TIME				;STARTS COUNTING FROM THE SAME VALUE


	BSF		GPIO,5				;SWITCH ON OUR LED INITIALLY

LED_LOOP:

	CALL	UPD_LED				;THIS IS WHERE WE GO AND CHECK TO SEE IF OUR TIMER HAS ROLLED OVER - IF IT HAS, IT WILL TOGGLE THE LED
								;OFF - THEN ON THEN OFF ETC AS LONG AS THE SWITCH IS STILL PRESSED

	BTFSS	GPIO,3				;TEST THE SWITCH INPUT - IF IT IS HIGH, I.E NO LONGER PRESSED, THEN THE NEXT INSTRUCTION IS
								;SUBSTITUTED WITH A "NOP" AUTOMATICALLY - IF IT IS LOW THEN OBVIOUSLY IT WON'T AND THE NEXT
								;INSTRUCTION WILL BE EXECUTED - SO WITH THAT IN MIND - IS THE SWITCH STILL BEING PRESSED ?

	GOTO	LED_LOOP			;YES - JUST STAY IN THIS LOOP UNTIL THE SWITCH IS NO LONGER BEING PRESSED
	
	BCF		GPIO,5				;SWITCH IS NO LONGER BEING PRESSED - SWITCH OFF THE LED AND

	GOTO	MAIN_LOOP			;START ALL OVER AGAIN
;************************************************************************
;************************************************************************	
;OBVIOUS - BUT WORTH A MENTION :
;************************************************************************

	END

;************************************************************************
